# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Examp Tel 2M Wavefront Fitting"""

import os
import sys
import matplotlib.pyplot as plt
import numpy as np
import pkg_resources

""" Looking for if KrakenOS is installed, if not, it assumes that
an folder downloaded from github is run"""

required = {'KrakenOS'}
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = required - installed

if missing:
    print("Not installed")
    import sys
    sys.path.append("../..")


import KrakenOS as Kos

# ______________________________________#

currentDirectory = os.getcwd()
sys.path.insert(1, currentDirectory + '/library')

# ______________________________________#

P_Obj = Kos.surf()
P_Obj.Rc = 0
P_Obj.Thickness = 1000
P_Obj.Glass = "AIR"
P_Obj.Diameter = 150.0 * 2.0
P_Obj.Drawing=1

# ______________________________________#


N0 = Kos.surf()
N0.Glass = "AIR"
N0.Diameter = 300
N0.Drawing=0

#############################
N1 = Kos.surf()
N1.Glass = "NULL"
N1.TiltZ = 0
N1.AxisMove = 1
N1.Drawing=0

#############################

N2 = Kos.surf()
N2.Glass = "NULL"
N2.TiltY = 20.7
N2.AxisMove = 1
N2.Drawing=0

###########################

G1 = Kos.surf()
G1.Thickness = -1000
G1.Glass = "MIRROR"
G1.Diameter = 1300
G1.Grating_D =1/0.3
G1.Diff_Ord = -1
G1.Grating_Angle = 90


P_Ima = Kos.surf()
P_Ima.Diameter = 498*4
P_Ima.Glass = "AIR"
P_Ima.Name = "Plano imagen"

# ______________________________________#

A = [P_Obj,N0, N1, N2, G1, P_Ima]
configuracion_1 = Kos.Setup()
Telescopio = Kos.system(A, configuracion_1)

# ______________________________________#
Telescopio = Kos.system(A, configuracion_1)
Rayos = Kos.raykeeper(Telescopio)

# ______________________________________#

tam = 9
rad = 100 / 2
tsis = len(A) - 1

# ______________________________________#



# ______________________________________#




# tet = 0.0
# pSource_0 = [0, 0, 0.0]
# dCos = [0.0, np.sin(np.deg2rad(tet)), np.cos(np.deg2rad(tet))]
# W = .55
# Telescopio.Trace(pSource_0, dCos, W)

# # print(Telescopio.Hit_x[-1], Telescopio.Hit_y[-1])

# print(Telescopio.OST_XYZ[-1])

# Rayos.push()






# for i in range(-tam, tam + 1):
for j in range(-tam, tam + 1):
    x_0 = (0 / tam) * rad
    y_0 = (j / tam) * rad
    r = np.sqrt((x_0 * x_0) + (y_0 * y_0))
    if r < rad:
        print(0)
        tet = 4.28*2
        pSource_0 = [x_0, y_0, 0.0]
        dCos = [0.0, -np.sin(np.deg2rad(tet)), -np.cos(np.deg2rad(tet))]
        print(dCos)
        print([-0.00314876, -0.07473735, -0.99719828])
        # dCos = [-0.00314876, -0.07473735, -0.99719828]
        W = .55
        Telescopio.Trace(pSource_0, dCos, W)
        Rayos.push()



for j in range(-tam, tam + 1):
    x_0 = (0 / tam) * rad
    y_0 = (j / tam) * rad
    r = np.sqrt((x_0 * x_0) + (y_0 * y_0))
    if r < rad:
        print(0)
        tet = -4.28*2
        pSource_0 = [x_0, y_0, 0.0]
        dCos = [0.0, -np.sin(np.deg2rad(tet)), -np.cos(np.deg2rad(tet))]
        print(dCos)
        print([-0.00314876, -0.07473735, -0.99719828])
        # dCos = [-0.00314876, -0.07473735, -0.99719828]
        W = .55
        Telescopio.Trace(pSource_0, dCos, W)
        Rayos.push()







for j in range(-tam, tam + 1):
    x_0 = (0 / tam) * rad
    y_0 = (j / tam) * rad
    r = np.sqrt((x_0 * x_0) + (y_0 * y_0))
    if r < rad:
        print(0)
        tet = 0.0
        pSource_0 = [x_0, y_0, 0.0]
        dCos = [0.0, -np.sin(np.deg2rad(tet)), -np.cos(np.deg2rad(tet))]
        print(dCos)
        print([-0.00314876, -0.07473735, -0.99719828])
        # dCos = [-0.00314876, -0.07473735, -0.99719828]
        W = .55
        Telescopio.Trace(pSource_0, dCos, W)
        Rayos.push()







# ______________________________________#
#

Kos.display3d(Telescopio, Rayos ,1)

# Kos.display2d(Telescopio, Rayos,1,0)

# print(Telescopio.EFFL)
# X, Y, Z, L, M, N = Rayos.pick(-1)

# # ______________________________________#

# plt.plot(X, Y, 'x')
# plt.xlabel('X')
# plt.ylabel('Y')
# plt.title('Spot Diagram')
# plt.axis('square')
# plt.show()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Examp Doublet Lens"""

import time
import numpy as np

import pkg_resources
""" Looking for if KrakenOS is installed, if not, it assumes that
an folder downloaded from github is run"""

required = {'KrakenOS'}
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = required - installed

if missing:
    print("Not installed")
    import sys
    sys.path.append("../..")


import KrakenOS as Kos

# ______________________________________#

start_time = time.time()


# ______________________________________#

def R_RMS(L, M, N, X, Y, delta_Z):
    cenX = np.mean(X)
    cenY = np.mean(Y)
    nX = X - cenX
    nY = Y - cenY
    x1 = ((L / N) * delta_Z) + nX
    y1 = ((M / N) * delta_Z) + nY
    R2 = ((x1 * x1) + (y1 * y1))
    R_RMS = np.sqrt(np.mean(R2))
    return R_RMS


# ______________________________________#

def DER_R_RMS(L, M, N, X, Y, delta_Z):
    h = 0.001
    f1 = R_RMS(L, M, N, X, Y, delta_Z + h)
    f2 = R_RMS(L, M, N, X, Y, delta_Z - h)
    der = (f1 - f2) / (2.0 * h)
    return der






OBJ=Kos.surf()
OBJ.Rc=0.0
OBJ.Thickness=10.0
OBJ.Glass="AIR"
OBJ.Diameter=56


STO=Kos.surf()
STO.Rc=0.0
STO.Thickness=10.0
STO.Glass="AIR"
STO.Diameter=56


s2=Kos.surf()
s2.Rc=91.66860362426125
s2.Thickness=2.16760007E+1
s2.Glass="BALKN3"
s2.Diameter=56
s2.DespX = 1.0
s2.AxisMove = 0

s3=Kos.surf()
s3.Rc=-60.18807811401379
s3.Thickness=3.48399646
s3.Glass="F4"
s3.Diameter=56
s3.DespY = 2.0
s3.AxisMove = 0

s4=Kos.surf()
s4.Rc=3972.618614963046
s4.Thickness=7.65093542E+1
s4.Glass="AIR"
s4.Diameter=56
s4.TiltX = 3.0
s4.AxisMove = 0

s5=Kos.surf()
s5.Rc=35.206739696593594
s5.Thickness=1.7360596E+1
s5.Glass="BK7"
s5.Diameter=48
s5.TiltY = 4.0
s5.AxisMove = 0

s6=Kos.surf()
s6.Rc=-58.825151255787866
s6.Thickness=3.48399646
s6.Glass="F2"
s6.Diameter=48
s6.DespX = -1
s6.AxisMove = 0

s7=Kos.surf()
s7.Rc=-166.14088338204223
s7.Thickness=1.93995258E+1
s7.Glass="AIR"
s7.Diameter=48
s7.DespY = -2
s7.AxisMove = 0


s8=Kos.surf()
s8.Rc=-27.78333066229023
s8.Thickness=1.96964573
s8.Glass="F2"
s8.Diameter = 32
s8.TiltX = -3
s8.AxisMove = 0


s9=Kos.surf()
s9.Rc=1111723.0516534068
s9.Thickness=15.1962995812
s9.Glass="AIR"
s9.Diameter=32

IMA=Kos.surf()
IMA.Rc=0.0
IMA.Thickness=0.0
IMA.Glass="AIR"
IMA.Diameter=13.98677

A = [OBJ, STO, s2, s3, s4, s5, s6, s7, s8, s9, IMA]

config_1 = Kos.Setup()

Petzval = Kos.system(A, config_1)
Rays = Kos.raykeeper(Petzval)

# ______________________________________#

W=0.4861
pSource_0 = [0, 0, 0]
dCos = [0., 0., 1.]
Petzval.Trace(pSource_0, dCos, W)
print(1.0)
for i in range(1,11):
    P=Petzval.OST_XYZ[-i]
    print(P[0])

print(" ")
for i in range(1,11):
    P=Petzval.OST_XYZ[-i]
    print(P[1])


print(type(P[1]))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Aug  2 12:04:14 2020

@author: joelherreravazquez
"""

from scipy import optimize
from scipy.optimize import fmin_cg

import pkg_resources
""" Looking for if KrakenOS is installed, if not, it assumes that
an folder downloaded from github is run"""

required = {'KrakenOS'}
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = required - installed

if missing:
    print("Not installed")
    import sys
    sys.path.append("../..")


import KrakenOS as Kos

import numpy as np
import matplotlib.pyplot as plt
import time


P_Obj=Kos.surf()
P_Obj.Rc=0
P_Obj.Thickness=3000+3.452200000000000E+003
P_Obj.Glass="AIR"
P_Obj.Diameter=6502.4
P_Obj.Drawing=0

Thickness=6178
M1=Kos.surf()
M1.Rc=-16256.0
M1.Thickness=-Thickness
M1.k=-1.
M1.Glass="MIRROR"
M1.Diameter=6502.4

M2=Kos.surf()
M2.Rc=-5150.974
M2.Thickness=Thickness+1851.28
M2.k=-2.6946
M2.Glass="MIRROR"

M2.Diameter=1714.5
M2.DespX=0
M2.AxisMove=0

P_Ima=Kos.surf()
P_Ima.Diameter=1000.0
P_Ima.Glass="AIR"
P_Ima.Name="Plano imagen"

A = [P_Obj, M1, M2, P_Ima]
configuracion_1 = Kos.Setup()
Telescope = Kos.system(A, configuracion_1)

# ______________________________________#

W=0.4861
pSource_0 = [6500/2., 0, 0]
dCos = [0., 0., 1.]
Telescope.Trace(pSource_0, dCos, W)
print(1.0)
for i in range(1,4):
    P=Telescope.OST_XYZ[-i]
    print(P[0])

print(" ")
for i in range(1,4):
    P=Telescope.OST_XYZ[-i]
    print(P[1])


print(type(P[1]))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Aug  2 12:04:14 2020

@author: joelherreravazquez
"""

from scipy import optimize
from scipy.optimize import fmin_cg

import pkg_resources
""" Looking for if KrakenOS is installed, if not, it assumes that
an folder downloaded from github is run"""

required = {'KrakenOS'}
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = required - installed

if missing:
    print("Not installed")
    import sys
    sys.path.append("../..")


import KrakenOS as Kos

import numpy as np
import matplotlib.pyplot as plt
import time


P_Obj=Kos.surf()
P_Obj.Rc=0
P_Obj.Thickness=3000+3.452200000000000E+003
P_Obj.Glass="AIR"
P_Obj.Diameter=6502.4
P_Obj.Drawing=0

Thickness=6178
M1=Kos.surf()
M1.Rc=-16256.0
M1.Thickness=-Thickness
M1.k=-1.
M1.Glass="MIRROR"
M1.Diameter=6502.4

M2=Kos.surf()
M2.Rc=-5150.974
M2.Thickness=Thickness+1851.28
M2.k=-2.6946
M2.Glass="MIRROR"
M2.Diameter=1714.5
M2.DespX=0
M2.AxisMove=0

P_Ima=Kos.surf()
P_Ima.Diameter=1000.0
P_Ima.Glass="AIR"
P_Ima.Name="Plano imagen"

A = [P_Obj, M1, M2, P_Ima]
configuracion_1 = Kos.Setup()
Telescope = Kos.system(A, configuracion_1)

# ______________________________________#

W = 0.4861
sup = 1
AperVal = 6500
AperType = "STOP"  # "STOP"
Pup = Kos.PupilCalc(Telescope, sup, W, AperType, AperVal)

Rays = Kos.raykeeper(Telescope)

# _________________________________________#

print("Radio pupila de entrada: ")
print(Pup.RadPupInp)
print("Posición pupila de entrada: ")
print(Pup.PosPupInp)
print("Rádio pupila de salida: ")
print(Pup.RadPupOut)
print("Posicion pupila de salida: ")
print(Pup.PosPupOut)
print("Posicion pupila de salida respecto al plano focal: ")
print(Pup.PosPupOutFoc)
print("Orientación pupila de salida")
print(Pup.DirPupSal)

print("Airy disk diameter focal distance (micrometers)")
print(Pup.FocusAiryRadius)


print("EFFL: ", Telescope.EFFL)
# _________________________________________#

[L, M, N] = Pup.DirPupSal
print(L, M, N)




Pup.Samp = 1
Pup.Ptype = "chief"
Pup.FieldType = "angle"
Pup.FieldX = 0.0
x, y, z, L, M, N = Pup.Pattern2Field()

pSource_0 = [x[0]+6500/2, y[0], z[0]]
dCos = [L[0], M[0], N[0]]
Telescope.Trace(pSource_0, dCos, W)
Rays.push()



print(" ")
print(" Coordenadas ")
print(Telescope.XYZ[0])
print(Telescope.XYZ[1])
print(Telescope.XYZ[2])
print(Telescope.XYZ[3][0])



print(" ")
print(" Cosenos directores ")
print(Telescope.LMN[0])
print(Telescope.LMN[1])
print(Telescope.LMN[2])





print(" ")
print(" Distancias ")
print(Telescope.DISTANCE[0])
print(Telescope.DISTANCE[1])
print(Telescope.DISTANCE[2])



# print(" ")
# print(" op ")
# print(" ")
# print(" Distancias opticas")
# print(Telescope.OP[0])
# print(Telescope.OP[1])
# print(Telescope.OP[2])
# print(Telescope.OP[3])
# print(Telescope.OP[4])
# print(Telescope.OP[5])
# print(Telescope.OP[6])
# print(Telescope.OP[7])
# print(Telescope.OP[8])
# print(Telescope.OP[9])
# print("Total = ", np.sum(Telescope.OP)- np.sum(Telescope.OP[0]))
# print("    ")

# print(Telescope.RP)
# print(Telescope.RS)
# print(Telescope.TP)
# print(Telescope.TS)
# print("Total trans : ")
# print(Telescope.TT)


















W = 0.4861
sup = 1
AperVal = 6500.
AperType = "EPD"
Pup = Kos.PupilCalc(Telescope, sup, W, AperType, AperVal)

# _________________________________________#

print("Radio pupila de entrada: ")
print(Pup.RadPupInp)
print("Posición pupila de entrada: ")
print(Pup.PosPupInp)
print("Rádio pupila de salida: ")
print(Pup.RadPupOut)
print("Posicion pupila de salida: ")
print(Pup.PosPupOut)
print("Posicion pupila de salida respecto al plano focal: ")
print(Pup.PosPupOutFoc)
print("Orientación pupila de salida")
print(Pup.DirPupSal)

print("Airy disk diameter focal distance (micrometers)")
print(Pup.FocusAiryRadius)


print("EFFL: ", Telescope.EFFL)
# _________________________________________#

[L, M, N] = Pup.DirPupSal
print(L, M, N)




# Pup.Samp = 1
# Pup.Ptype = "chief"
# Pup.FieldType = "angle"
# Pup.FieldX = 1.0
# x, y, z, L, M, N = Pup.Pattern2Field()

# pSource_0 = [x[0], y[0], z[0]]
# dCos = [L[0], M[0], N[0]]
# Telescope.Trace(pSource_0, dCos, W)
# Rays.push()



# print(" ")
# print(" Coordenadas ")
# print(Telescope.XYZ[0])
# print(Telescope.XYZ[1])
# print(Telescope.XYZ[2])
# print(Telescope.XYZ[3])
# print(Telescope.XYZ[4])
# print(Telescope.XYZ[5])
# print(Telescope.XYZ[6])
# print(Telescope.XYZ[7])
# print(Telescope.XYZ[8])
# print(Telescope.XYZ[9])
# XYZ= Telescope.XYZ[10]
# print(XYZ[0], XYZ[1], XYZ[2])


# print(" ")
# print(" Cosenos directores ")
# print(Telescope.LMN[0])
# print(Telescope.LMN[1])
# print(Telescope.LMN[2])
# print(Telescope.LMN[3])
# print(Telescope.LMN[4])
# print(Telescope.LMN[5])
# print(Telescope.LMN[6])
# print(Telescope.LMN[7])
# print(Telescope.LMN[8])
# print(Telescope.LMN[9])
# LMN = Telescope.LMN[9]
# print(LMN[0], LMN[1], LMN[2])





# print(" ")
# print(" Distancias ")
# print(Telescope.DISTANCE[0])
# print(Telescope.DISTANCE[1])
# print(Telescope.DISTANCE[2])
# print(Telescope.DISTANCE[3])
# print(Telescope.DISTANCE[4])
# print(Telescope.DISTANCE[5])
# print(Telescope.DISTANCE[6])
# print(Telescope.DISTANCE[7])
# print(Telescope.DISTANCE[8])
# print(Telescope.DISTANCE[9])

# print(" ")
# print(" op ")
# print(" ")
# print(" Distancias opticas")
# print(Telescope.OP[0])
# print(Telescope.OP[1])
# print(Telescope.OP[2])
# print(Telescope.OP[3])
# print(Telescope.OP[4])
# print(Telescope.OP[5])
# print(Telescope.OP[6])
# print(Telescope.OP[7])
# print(Telescope.OP[8])
# print(Telescope.OP[9])
# print("Total = ", np.sum(Telescope.OP)- np.sum(Telescope.OP[0]))
# print("    ")

# print(Telescope.RP)
# print(Telescope.RS)
# print(Telescope.TP)
# print(Telescope.TS)
# print("Total trans : ")
# print(Telescope.TT)


# _________________________________________#

Pup.Samp = 15
Pup.Ptype = "hexapolar"
Pup.FieldType = "angle"
Pup.FieldX =0.5

# x, y, z, L, M, N = Pup.Pattern2Field()
# for i in range(0, len(x)):
#     pSource_0 = [x[i], y[i], z[i]]
#     dCos = [L[i], M[i], N[i]]
#     Telescope.Trace(pSource_0, dCos, W)
#     Rays.push()

# Kos.display2d(Telescope, Rays, 1, 0 )



X, Y, Z, P2V = Kos.Phase(Pup)


"""Indicamos el grado de expanción para los polinomios de Zernike"""
NC = 35

"""Generamos un arreglo numpy conlas mismas dimensiones de la expanción definida"""
A = np.ones(NC)

"""Calculamos los polinomios de Zernike con la fase calculada y el numero de
  elementos deseados en la expanción, Zcoef son los coeficientes en longitudes de
  onda, Mat es la expreción matematica de Zeidel para dicho coeficiente,
  esto con fines ilustrativos, w_rms es el error del ajuste"""

Zcoef, Mat, RMS2Chief, RMS2Centroid, FITTINGERROR = Kos.Zernike_Fitting(X, Y, Z, A)

"""" Se despliegan los resultados """

for i in range(0, NC):
    print("z", i + 1, "  ", "{0:.8f}".format(float(Zcoef[i])), ":", Mat[i])

# # ______________________________________#
print("(RMS) Fitting error: ", FITTINGERROR)
print(RMS2Chief, "RMS(to chief) From fitted coefficents")
print(RMS2Centroid, "RMS(to centroid) From fitted coefficents")
